package org.mercury_im.messenger.ui.chat;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import org.mercury_im.messenger.R;
import org.mercury_im.messenger.persistence.model.MessageModel;

import java.util.ArrayList;
import java.util.List;

public class ChatRecyclerViewAdapter extends RecyclerView.Adapter<ChatRecyclerViewAdapter.ChatItemViewHolder> {

    private List<MessageModel> messages = new ArrayList<>();

    public ChatRecyclerViewAdapter() {

    }

    public void updateMessages(List<MessageModel> messages) {
        this.messages.clear();
        this.messages.addAll(messages);
        notifyDataSetChanged();
    }

    @NonNull
    @Override
    public ChatItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ChatItemViewHolder(LayoutInflater.from(parent.getContext()).inflate(viewType, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ChatItemViewHolder holder, int position) {
        MessageModel message = messages.get(position);
        MessageBackgroundDrawable background = getBackgroundDrawable(position);
        if (message.isIncoming()) {
            holder.body.setBackgroundResource(background.getIn());
        } else {
            holder.body.setBackgroundResource(background.getOut());
        }
        holder.body.setText(message.getBody());
    }

    @Override
    public int getItemViewType(int position) {
        if (messages.get(position).isIncoming()) {
            return R.layout.view_message_text_in;
        } else {
            return R.layout.view_message_text_out;
        }
    }

    @Override
    public int getItemCount() {
        return messages.size();
    }

    public class ChatItemViewHolder extends RecyclerView.ViewHolder {

        private TextView body;

        public ChatItemViewHolder(@NonNull View itemView) {
            super(itemView);

            body = itemView.findViewById(R.id.msg_body);
        }
    }

    enum MessageBackgroundDrawable {
        FIRST(R.drawable.bg_msg_in_first, R.drawable.bg_msg_out_first),
        MID(R.drawable.bg_msg_in_mid, R.drawable.bg_msg_out_mid),
        LAST(R.drawable.bg_msg_in_last, R.drawable.bg_msg_out_last),
        SINGLE(R.drawable.bg_msg_single, R.drawable.bg_msg_single),
        ;

        private final int in, out;

        MessageBackgroundDrawable(int in, int out) {
            this.in = in;
            this.out = out;
        }

        public int getIn() {
            return in;
        }

        public int getOut() {
            return out;
        }
    }

    public MessageBackgroundDrawable getBackgroundDrawable(int pos) {
        MessageModel m = messages.get(pos);
        MessageModel b = pos != 0 ? messages.get(pos - 1) : null;
        MessageModel a = pos != messages.size() - 1 ? messages.get(pos + 1) : null;

        if (b == null || b.isIncoming() != m.isIncoming()) {
            if (a == null || a.isIncoming() != m.isIncoming()) {
                return MessageBackgroundDrawable.SINGLE;
            } else {
                return MessageBackgroundDrawable.FIRST;
            }
        } else {
            if (a == null || a.isIncoming() != m.isIncoming()) {
                return MessageBackgroundDrawable.LAST;
            } else {
                return MessageBackgroundDrawable.MID;
            }
        }
    }
}
