package org.mercury_im.messenger.ui.roster;

import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityOptionsCompat;
import androidx.recyclerview.widget.RecyclerView;

import org.jivesoftware.smackx.colors.ConsistentColor;
import org.jxmpp.jid.EntityBareJid;
import org.mercury_im.messenger.R;
import org.mercury_im.messenger.persistence.room.model.RoomContactModel;
import org.mercury_im.messenger.ui.chat.ChatActivity;
import org.mercury_im.messenger.util.ColorUtil;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import de.hdodenhof.circleimageview.CircleImageView;

public class RosterRecyclerViewAdapter
        extends RecyclerView.Adapter<RosterRecyclerViewAdapter.RosterItemViewHolder> {

    private List<RoomContactModel> entryModelList;

    public RosterRecyclerViewAdapter(List<RoomContactModel> entryModelList) {
        this.entryModelList = entryModelList;
    }

    @NonNull
    @Override
    public RosterItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new RosterItemViewHolder(parent.getContext(), LayoutInflater.from(parent.getContext())
                .inflate(R.layout.recycler_view_item_roster_entry, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull RosterItemViewHolder holder, int position) {
        RoomContactModel model = entryModelList.get(position);
        holder.bind(model);
    }

    @Override
    public int getItemCount() {
        return entryModelList.size();
    }

    public void setItems(List<RoomContactModel> rosterEntryModels) {
        this.entryModelList = rosterEntryModels;
        notifyDataSetChanged();
    }

    public class RosterItemViewHolder extends RecyclerView.ViewHolder {

        private View view;

        @BindView(R.id.roster_entry__jid)
        TextView jidView;

        @BindView(R.id.roster_entry__nickname)
        TextView nicknameView;

        @BindView(R.id.roster_entry__avatar)
        CircleImageView avatarView;

        Context context;

        public RosterItemViewHolder(Context context, View itemView) {
            super(itemView);
            this.context = context;
            this.view = itemView;
            ButterKnife.bind(this, view);
        }

        void bind(RoomContactModel contactModel) {
            String nick = contactModel.getNickname();
            nicknameView.setText(nick != null ? nick : "");
            EntityBareJid jid = contactModel.getEntity().getJid();
            jidView.setText(jid.toString());
            jidView.setTextColor(ColorUtil.consistentColor(jid.toString(), new ConsistentColor.ConsistentColorSettings(ConsistentColor.Deficiency.none)));
            view.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {

                    Intent intent = new Intent(context, ChatActivity.class);
                    intent.putExtra("JID", jid.toString());
                    intent.putExtra("ACCOUNT", contactModel.getEntity().getAccountId());

                    // Animation
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                        ActivityOptionsCompat options = ActivityOptionsCompat
                                .makeSceneTransitionAnimation((AppCompatActivity) context, avatarView, "avatar");
                        context.startActivity(intent, options.toBundle());
                    } else {
                        context.startActivity(intent);
                    }
                }
            });
        }
    }
}
