package org.mercury_im.messenger.persistence.room.model;

import androidx.room.ColumnInfo;
import androidx.room.Entity;
import androidx.room.ForeignKey;
import androidx.room.Index;
import androidx.room.PrimaryKey;

import org.mercury_im.messenger.persistence.model.ChatModel;

import static androidx.room.ForeignKey.CASCADE;
import static org.mercury_im.messenger.persistence.room.model.RoomChatModel.KEY_ID;
import static org.mercury_im.messenger.persistence.room.model.RoomChatModel.KEY_ENTITY;
import static org.mercury_im.messenger.persistence.room.model.RoomChatModel.TABLE;

@Entity(tableName = TABLE,
        indices = {
                @Index(KEY_ID), @Index(KEY_ENTITY)
        },
        foreignKeys = {
                @ForeignKey(entity = RoomEntityModel.class,
                        parentColumns = RoomEntityModel.KEY_ID,
                        childColumns = KEY_ENTITY,
                        onDelete = CASCADE)
        })
public class RoomChatModel implements ChatModel {

    public static final String TABLE = "chats";
    public static final String KEY_ID = "pk_chat_id";

    public static final String KEY_ENTITY = "fk_entity_id";
    public static final String KEY_ACTIVE = "active";

    @PrimaryKey(autoGenerate = true)
    @ColumnInfo(name = KEY_ID)
    private long id;

    @ColumnInfo(name = KEY_ENTITY)
    private long peerEntityId;

    @ColumnInfo(name = KEY_ACTIVE)
    private boolean isActive;

    @Override
    public long getId() {
        return id;
    }

    @Override
    public void setId(long id) {
        this.id = id;
    }

    @Override
    public long getPeerEntityId() {
        return peerEntityId;
    }

    @Override
    public void setPeerEntityId(long id) {
        this.peerEntityId = id;
    }

    @Override
    public boolean isActive() {
        return isActive;
    }

    @Override
    public void setActive(boolean active) {
        this.isActive = active;
    }
}
